from enum import Enum

class Action(Enum):
    NEUTRAL = 0
    STAND = 1
    FORWARD_WALK = 2
    DASH = 3
    BACK_STEP = 4
    CROUCH = 5
    JUMP = 6
    FOR_JUMP = 7
    BACK_JUMP = 8
    AIR = 9
    STAND_GUARD = 10
    CROUCH_GUARD = 11
    AIR_GUARD = 12
    STAND_GUARD_RECOV = 13
    CROUCH_GUARD_RECOV = 14
    AIR_GUARD_RECOV = 15
    STAND_RECOV = 16
    CROUCH_RECOV = 17
    AIR_RECOV = 18
    CHANGE_DOWN = 19
    DOWN = 20
    RISE = 21
    LANDING = 22
    THROW_A = 23
    THROW_B = 24
    THROW_HIT = 25
    THROW_SUFFER = 26
    STAND_A = 27
    STAND_B = 28
    CROUCH_A = 29
    CROUCH_B = 30
    AIR_A = 31
    AIR_B = 32
    AIR_DA = 33
    AIR_DB = 34
    STAND_FA = 35
    STAND_FB = 36
    CROUCH_FA = 37
    CROUCH_FB = 38
    AIR_FA = 39
    AIR_FB = 40
    AIR_UA = 41
    AIR_UB = 42
    STAND_D_DF_FA = 43
    STAND_D_DF_FB = 44
    STAND_F_D_DFA = 45
    STAND_F_D_DFB = 46
    STAND_D_DB_BA = 47
    STAND_D_DB_BB = 48
    AIR_D_DF_FA = 49
    AIR_D_DF_FB = 50
    AIR_F_D_DFA = 51
    AIR_F_D_DFB = 52
    AIR_D_DB_BA = 53
    AIR_D_DB_BB = 54
    STAND_D_DF_FC = 55



import numpy as np

class GameData:
    def __init__(self, game_data):
        self.max_hps = np.array(game_data.max_hps, dtype=np.int32)
        self.max_energies = np.array(game_data.max_energies, dtype=np.int32)
        self.character_names = (game_data.character_names[0], game_data.character_names[1])
        self.ai_names = (game_data.ai_names[0], game_data.ai_names[1])



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: message.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import enum_pb2 as enum__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rmessage.proto\x12\x07service\x1a\nenum.proto\"G\n\x0bGrpcHitArea\x12\x0c\n\x04left\x18\x01 \x01(\x05\x12\r\n\x05right\x18\x02 \x01(\x05\x12\x0b\n\x03top\x18\x03 \x01(\x05\x12\x0e\n\x06\x62ottom\x18\x04 \x01(\x05\"\x9c\x04\n\x0eGrpcAttackData\x12.\n\x10setting_hit_area\x18\x01 \x01(\x0b\x32\x14.service.GrpcHitArea\x12\x17\n\x0fsetting_speed_x\x18\x02 \x01(\x05\x12\x17\n\x0fsetting_speed_y\x18\x03 \x01(\x05\x12.\n\x10\x63urrent_hit_area\x18\x04 \x01(\x0b\x32\x14.service.GrpcHitArea\x12\x15\n\rcurrent_frame\x18\x05 \x01(\x05\x12\x15\n\rplayer_number\x18\x06 \x01(\x08\x12\x0f\n\x07speed_x\x18\x07 \x01(\x05\x12\x0f\n\x07speed_y\x18\x08 \x01(\x05\x12\x10\n\x08start_up\x18\t \x01(\x05\x12\x0e\n\x06\x61\x63tive\x18\n \x01(\x05\x12\x12\n\nhit_damage\x18\x0b \x01(\x05\x12\x14\n\x0cguard_damage\x18\x0c \x01(\x05\x12\x18\n\x10start_add_energy\x18\r \x01(\x05\x12\x16\n\x0ehit_add_energy\x18\x0e \x01(\x05\x12\x18\n\x10guard_add_energy\x18\x0f \x01(\x05\x12\x13\n\x0bgive_energy\x18\x10 \x01(\x05\x12\x10\n\x08impact_x\x18\x11 \x01(\x05\x12\x10\n\x08impact_y\x18\x12 \x01(\x05\x12\x18\n\x10give_guard_recov\x18\x13 \x01(\x05\x12\x13\n\x0b\x61ttack_type\x18\x14 \x01(\x05\x12\x11\n\tdown_prop\x18\x15 \x01(\x08\x12\x15\n\ris_projectile\x18\x16 \x01(\x08\"\xf1\x03\n\x11GrpcCharacterData\x12\x15\n\rplayer_number\x18\x01 \x01(\x08\x12\n\n\x02hp\x18\x02 \x01(\x05\x12\x0e\n\x06\x65nergy\x18\x03 \x01(\x05\x12\t\n\x01x\x18\x04 \x01(\x05\x12\t\n\x01y\x18\x05 \x01(\x05\x12\x0c\n\x04left\x18\x06 \x01(\x05\x12\r\n\x05right\x18\x07 \x01(\x05\x12\x0b\n\x03top\x18\x08 \x01(\x05\x12\x0e\n\x06\x62ottom\x18\t \x01(\x05\x12\x0f\n\x07speed_x\x18\n \x01(\x05\x12\x0f\n\x07speed_y\x18\x0b \x01(\x05\x12!\n\x05state\x18\x0c \x01(\x0e\x32\x12.service.GrpcState\x12#\n\x06\x61\x63tion\x18\r \x01(\x0e\x32\x13.service.GrpcAction\x12\r\n\x05\x66ront\x18\x0e \x01(\x08\x12\x0f\n\x07\x63ontrol\x18\x0f \x01(\x08\x12,\n\x0b\x61ttack_data\x18\x10 \x01(\x0b\x32\x17.service.GrpcAttackData\x12\x17\n\x0fremaining_frame\x18\x11 \x01(\x05\x12\x13\n\x0bhit_confirm\x18\x12 \x01(\x08\x12\x16\n\x0egraphic_size_x\x18\x13 \x01(\x05\x12\x16\n\x0egraphic_size_y\x18\x14 \x01(\x05\x12\x18\n\x10graphic_adjust_x\x18\x15 \x01(\x05\x12\x11\n\thit_count\x18\x16 \x01(\x05\x12\x16\n\x0elast_hit_frame\x18\x17 \x01(\x05\"\xcd\x01\n\rGrpcFrameData\x12\x32\n\x0e\x63haracter_data\x18\x01 \x03(\x0b\x32\x1a.service.GrpcCharacterData\x12\x1c\n\x14\x63urrent_frame_number\x18\x02 \x01(\x05\x12\x15\n\rcurrent_round\x18\x03 \x01(\x05\x12\x30\n\x0fprojectile_data\x18\x04 \x03(\x0b\x32\x17.service.GrpcAttackData\x12\x12\n\nempty_flag\x18\x05 \x01(\x08\x12\r\n\x05\x66ront\x18\x06 \x03(\x08\"J\n\x0bGrpcFftData\x12\x1a\n\x12real_data_as_bytes\x18\x01 \x01(\x0c\x12\x1f\n\x17imaginary_data_as_bytes\x18\x02 \x01(\x0c\"\'\n\x0eGrpcScreenData\x12\x15\n\rdisplay_bytes\x18\x01 \x01(\x0c\"u\n\rGrpcAudioData\x12\x19\n\x11raw_data_as_bytes\x18\x01 \x01(\x0c\x12&\n\x08\x66\x66t_data\x18\x02 \x03(\x0b\x32\x14.service.GrpcFftData\x12!\n\x19spectrogram_data_as_bytes\x18\x03 \x01(\x0c\"`\n\x0cGrpcGameData\x12\x0f\n\x07max_hps\x18\x01 \x03(\x05\x12\x14\n\x0cmax_energies\x18\x02 \x03(\x05\x12\x17\n\x0f\x63haracter_names\x18\x03 \x03(\t\x12\x10\n\x08\x61i_names\x18\x04 \x03(\t\"V\n\x0fGrpcRoundResult\x12\x15\n\rcurrent_round\x18\x01 \x01(\x05\x12\x15\n\rremaining_hps\x18\x02 \x03(\x05\x12\x15\n\relapsed_frame\x18\x03 \x01(\x05\"V\n\x07GrpcKey\x12\t\n\x01\x41\x18\x01 \x01(\x08\x12\t\n\x01\x42\x18\x02 \x01(\x08\x12\t\n\x01\x43\x18\x03 \x01(\x08\x12\t\n\x01U\x18\x04 \x01(\x08\x12\t\n\x01R\x18\x05 \x01(\x08\x12\t\n\x01\x44\x18\x06 \x01(\x08\x12\t\n\x01L\x18\x07 \x01(\x08\x42\x1e\n\x06protocB\x0cMessageProtoP\x00\xa2\x02\x03HLWb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'message_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006protocB\014MessageProtoP\000\242\002\003HLW'
  _GRPCHITAREA._serialized_start=38
  _GRPCHITAREA._serialized_end=109
  _GRPCATTACKDATA._serialized_start=112
  _GRPCATTACKDATA._serialized_end=652
  _GRPCCHARACTERDATA._serialized_start=655
  _GRPCCHARACTERDATA._serialized_end=1152
  _GRPCFRAMEDATA._serialized_start=1155
  _GRPCFRAMEDATA._serialized_end=1360
  _GRPCFFTDATA._serialized_start=1362
  _GRPCFFTDATA._serialized_end=1436
  _GRPCSCREENDATA._serialized_start=1438
  _GRPCSCREENDATA._serialized_end=1477
  _GRPCAUDIODATA._serialized_start=1479
  _GRPCAUDIODATA._serialized_end=1596
  _GRPCGAMEDATA._serialized_start=1598
  _GRPCGAMEDATA._serialized_end=1694
  _GRPCROUNDRESULT._serialized_start=1696
  _GRPCROUNDRESULT._serialized_end=1782
  _GRPCKEY._serialized_start=1784
  _GRPCKEY._serialized_end=1870
# @@protoc_insertion_point(module_scope)



from .ai_interface import AIInterface
from .struct.enumerate.flag import Flag
from .struct import Key, FrameData, ScreenData, AudioData, GameData, RoundResult
from .util import convert_key
from .protoc import service_pb2, service_pb2_grpc
from threading import Thread

class AIController(Thread):
    def __init__(self, stub: service_pb2_grpc.ServiceStub, ai: AIInterface, player_number: bool):
        super(AIController, self).__init__()
        self.stub = stub
        self.ai = ai
        self.player_number = player_number
    
    def initialize_rpc(self):
        request = service_pb2.InitializeRequest(player_number=self.player_number, player_name=self.ai.name(), is_blind=self.ai.is_blind())
        response = self.stub.Initialize(request)
        self.player_uuid: str = response.player_uuid

    def participate_rpc(self):
        request = service_pb2.ParticipateRequest(player_uuid=self.player_uuid)
        return self.stub.Participate(request)
    
    def input_rpc(self, key: Key):
        self.stub.Input(service_pb2.PlayerInput(player_uuid=self.player_uuid, input_key=convert_key(key)))
    
    def run(self):
        self.initialize_rpc()
        for state in self.participate_rpc():
            flag = Flag(state.state_flag)
            if flag is Flag.INITIALIZE:
                self.ai.initialize(GameData(state.game_data), self.player_number)
            elif flag is Flag.PROCESSING:
                self.ai.get_information(FrameData(state.frame_data), state.is_control, None if not state.HasField("non_delay_frame_data") else FrameData(state.non_delay_frame_data))

                if state.HasField("screen_data"):
                    self.ai.get_screen_data(ScreenData(state.screen_data))

                if state.HasField("audio_data"):
                    self.ai.get_audio_data(AudioData(state.audio_data))
                    
                self.ai.processing()
                self.input_rpc(self.ai.input())
            elif flag is Flag.ROUND_END:
                self.ai.round_end(RoundResult(state.round_result))
            elif flag is Flag.GAME_END:
                self.ai.round_end(RoundResult(state.round_result))
                self.ai.game_end()



from .hit_area import HitArea

class AttackData:
    def __init__(self, attack_data):
        self.setting_hit_area = HitArea(attack_data.setting_hit_area)
        self.setting_speed_x: int = attack_data.setting_speed_x
        self.setting_speed_y: int = attack_data.setting_speed_y
        self.current_hit_area = HitArea(attack_data.current_hit_area)
        self.current_frame: int = attack_data.current_frame
        self.player_number: bool = attack_data.player_number
        self.speed_x: int = attack_data.speed_x
        self.speed_y: int = attack_data.speed_y
        self.start_up: int = attack_data.start_up
        self.active: int = attack_data.active
        self.hit_damage: int = attack_data.hit_damage
        self.guard_damage: int = attack_data.guard_damage
        self.start_add_energy: int = attack_data.start_add_energy
        self.hit_add_energy: int = attack_data.hit_add_energy
        self.guard_add_energy: int = attack_data.guard_add_energy
        self.give_energy: int = attack_data.give_energy
        self.impact_x: int = attack_data.impact_x
        self.impact_y: int = attack_data.impact_y
        self.give_guard_recov: int = attack_data.give_guard_recov
        self.attack_type: int = attack_data.attack_type
        self.down_prop: bool = attack_data.down_prop
        self.is_projectile: bool = attack_data.is_projectile



from .protoc import service_pb2, service_pb2_grpc
from .ai_controller import AIController
from .ai_interface import AIInterface
from .util import load_ai
import grpc

class Gateway:
    def __init__(self, host='127.0.0.1', port=50051):
        self.host = host
        self.port = port
        self.channel = grpc.insecure_channel(f'{host}:{port}')
        self.stub = service_pb2_grpc.ServiceStub(self.channel)
        self.registered_agents: 'dict[str, AIInterface]' = dict()
        self.agents: 'list[AIInterface]' = [None, None]
        self.ais: 'list[AIController]' = [None, None]
    
    def load_agent(self, ai_names: 'list[str]'):
        for i, ai_name in enumerate(ai_names):
            if ai_name:
                self.agents[i] = load_ai(ai_name)
    
    def register_ai(self, name: str, agent: AIInterface):
        self.registered_agents[name] = agent

    def run_game(self, characters: 'list[str]', agents: 'list[str]', game_number: int):
        for i in range(2):
            if agents[i] == 'Sandbox':
                agents[i] = None
            elif agents[i] in self.registered_agents:
                self.agents[i] = self.registered_agents[agents[i]]
        self.stub.RunGame(service_pb2.RunGameRequest(character_1=characters[0], character_2=characters[1], player_1=agents[0], player_2=agents[1], game_number=game_number))
        self.start_ai()
    
    def start_ai(self):
        for i, agent in enumerate(self.agents):
            if agent:
                self.ais[i] = AIController(self.stub, agent, i == 0)
                self.ais[i].start()
        for ai in [x for x in self.ais if x]:
            ai.join()
    
    def close(self):
        self.channel.close()



from .fft_data import FftData
from array import array
import numpy as np

class AudioData:
    def __init__(self, audio_data):
        self.raw_data_as_bytes: bytes = audio_data.raw_data_as_bytes
        self.fft_data = [FftData(x) for x in audio_data.fft_data]
        self.spectrogram_data_as_bytes: bytes = audio_data.spectrogram_data_as_bytes
    
    def transform_data(self):
        self.raw_data = np.reshape(array('f', self.raw_data_as_bytes), (2, 1024))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from . import service_pb2 as service__pb2


class ServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.RunGame = channel.unary_unary(
                '/service.Service/RunGame',
                request_serializer=service__pb2.RunGameRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.Spectate = channel.unary_stream(
                '/service.Service/Spectate',
                request_serializer=service__pb2.SpectateRequest.SerializeToString,
                response_deserializer=service__pb2.SpectatorGameState.FromString,
                )
        self.Initialize = channel.unary_unary(
                '/service.Service/Initialize',
                request_serializer=service__pb2.InitializeRequest.SerializeToString,
                response_deserializer=service__pb2.InitializeResponse.FromString,
                )
        self.Participate = channel.unary_stream(
                '/service.Service/Participate',
                request_serializer=service__pb2.ParticipateRequest.SerializeToString,
                response_deserializer=service__pb2.PlayerGameState.FromString,
                )
        self.Input = channel.unary_unary(
                '/service.Service/Input',
                request_serializer=service__pb2.PlayerInput.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )


class ServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def RunGame(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Spectate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Initialize(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Participate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Input(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'RunGame': grpc.unary_unary_rpc_method_handler(
                    servicer.RunGame,
                    request_deserializer=service__pb2.RunGameRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'Spectate': grpc.unary_stream_rpc_method_handler(
                    servicer.Spectate,
                    request_deserializer=service__pb2.SpectateRequest.FromString,
                    response_serializer=service__pb2.SpectatorGameState.SerializeToString,
            ),
            'Initialize': grpc.unary_unary_rpc_method_handler(
                    servicer.Initialize,
                    request_deserializer=service__pb2.InitializeRequest.FromString,
                    response_serializer=service__pb2.InitializeResponse.SerializeToString,
            ),
            'Participate': grpc.unary_stream_rpc_method_handler(
                    servicer.Participate,
                    request_deserializer=service__pb2.ParticipateRequest.FromString,
                    response_serializer=service__pb2.PlayerGameState.SerializeToString,
            ),
            'Input': grpc.unary_unary_rpc_method_handler(
                    servicer.Input,
                    request_deserializer=service__pb2.PlayerInput.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'service.Service', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Service(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def RunGame(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/service.Service/RunGame',
            service__pb2.RunGameRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Spectate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/service.Service/Spectate',
            service__pb2.SpectateRequest.SerializeToString,
            service__pb2.SpectatorGameState.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Initialize(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/service.Service/Initialize',
            service__pb2.InitializeRequest.SerializeToString,
            service__pb2.InitializeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Participate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/service.Service/Participate',
            service__pb2.ParticipateRequest.SerializeToString,
            service__pb2.PlayerGameState.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Input(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/service.Service/Input',
            service__pb2.PlayerInput.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from .attack_data import AttackData
from .character_data import CharacterData
import numpy as np

class FrameData:
    def __init__(self, frame_data=None):
        if not frame_data:
            self.character_data = [None, None]
            self.current_frame_number = -1
            self.current_round = -1
            self.projectile_data = []
            self.empty_flag = True
            self.front = [False, False]
        else:
            self.character_data = [CharacterData(x) for x in frame_data.character_data]
            self.current_frame_number: int = frame_data.current_frame_number
            self.current_round: int = frame_data.current_round
            self.projectile_data = [AttackData(x) for x in frame_data.projectile_data]
            self.empty_flag: bool = frame_data.empty_flag
            self.front = np.array(frame_data.front, dtype=np.bool8)

    def is_front(self, player: bool):
        return self.front[0 if player else 1]
    
    def get_character(self, player: bool):
        return self.character_data[0 if player else 1]



class HitArea:
    def __init__(self, hit_area):
        self.left: int = hit_area.left
        self.right: int = hit_area.right
        self.top: int = hit_area.top
        self.bottom: int = hit_area.bottom\
    


import numpy as np

class RoundResult:
    def __init__(self, round_result):
        self.current_round = round_result.current_round
        self.remaining_hps = np.array(round_result.remaining_hps, dtype=np.int32)
        self.elapsed_frame = round_result.elapsed_frame



from .struct import Key
from .ai_interface import AIInterface
from .protoc import message_pb2

def convert_key(key: Key) -> message_pb2.GrpcKey:
    return message_pb2.GrpcKey(A=key.A, B=key.B, C=key.C, U=key.U, D=key.D, L=key.L, R=key.R)

def load_ai(ai_path: str) -> AIInterface:
    path = ai_path.split('.')
    module_name, class_name = path[0], path[int(len(path) == 2)]
    return getattr(__import__(module_name), class_name)()



from .attack_data import AttackData
from .enumerate.action import Action
from .enumerate.state import State

class CharacterData:
    def __init__(self, character_data):
        self.player_number: bool = character_data.player_number
        self.hp: int = character_data.hp
        self.energy: int = character_data.energy
        self.x: int = character_data.x
        self.y: int = character_data.y
        self.left: int = character_data.left
        self.right: int = character_data.right
        self.top: int = character_data.top
        self.bottom: int = character_data.bottom
        self.speed_x: int = character_data.speed_x
        self.speed_y: int = character_data.speed_y
        self.state = State(character_data.state)
        self.action = Action(character_data.action)
        self.front: bool = character_data.front
        self.control: bool = character_data.control
        self.attack_data = AttackData(character_data.attack_data)
        self.remaining_frame: int = character_data.remaining_frame
        self.hit_confirm: bool = character_data.hit_confirm
        self.graphic_size_x: int = character_data.graphic_size_x
        self.graphic_size_y: int = character_data.graphic_size_y
        self.graphic_adjust_x: int = character_data.graphic_adjust_x
        self.hit_count: int = character_data.hit_count
        self.last_hit_frame: int = character_data.last_hit_frame
    


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from . import enum_pb2 as enum__pb2
from . import message_pb2 as message__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rservice.proto\x12\x07service\x1a\x1bgoogle/protobuf/empty.proto\x1a\nenum.proto\x1a\rmessage.proto\"s\n\x0eRunGameRequest\x12\x13\n\x0b\x63haracter_1\x18\x01 \x01(\t\x12\x13\n\x0b\x63haracter_2\x18\x02 \x01(\t\x12\x10\n\x08player_1\x18\x03 \x01(\t\x12\x10\n\x08player_2\x18\x04 \x01(\t\x12\x13\n\x0bgame_number\x18\x05 \x01(\x05\"\x11\n\x0fSpectateRequest\"\xf1\x01\n\x12SpectatorGameState\x12%\n\nstate_flag\x18\x01 \x01(\x0e\x32\x11.service.GrpcFlag\x12*\n\nframe_data\x18\x02 \x01(\x0b\x32\x16.service.GrpcFrameData\x12,\n\x0bscreen_data\x18\x03 \x01(\x0b\x32\x17.service.GrpcScreenData\x12*\n\naudio_data\x18\x04 \x01(\x0b\x32\x16.service.GrpcAudioData\x12.\n\x0cround_result\x18\x05 \x01(\x0b\x32\x18.service.GrpcRoundResult\"Q\n\x11InitializeRequest\x12\x15\n\rplayer_number\x18\x01 \x01(\x08\x12\x13\n\x0bplayer_name\x18\x02 \x01(\t\x12\x10\n\x08is_blind\x18\x03 \x01(\x08\")\n\x12InitializeResponse\x12\x13\n\x0bplayer_uuid\x18\x01 \x01(\t\")\n\x12ParticipateRequest\x12\x13\n\x0bplayer_uuid\x18\x01 \x01(\t\"\xe2\x02\n\x0fPlayerGameState\x12%\n\nstate_flag\x18\x01 \x01(\x0e\x32\x11.service.GrpcFlag\x12\x12\n\nis_control\x18\x02 \x01(\x08\x12*\n\nframe_data\x18\x03 \x01(\x0b\x32\x16.service.GrpcFrameData\x12\x34\n\x14non_delay_frame_data\x18\x04 \x01(\x0b\x32\x16.service.GrpcFrameData\x12,\n\x0bscreen_data\x18\x05 \x01(\x0b\x32\x17.service.GrpcScreenData\x12*\n\naudio_data\x18\x06 \x01(\x0b\x32\x16.service.GrpcAudioData\x12(\n\tgame_data\x18\x07 \x01(\x0b\x32\x15.service.GrpcGameData\x12.\n\x0cround_result\x18\x08 \x01(\x0b\x32\x18.service.GrpcRoundResult\"G\n\x0bPlayerInput\x12\x13\n\x0bplayer_uuid\x18\x01 \x01(\t\x12#\n\tinput_key\x18\x02 \x01(\x0b\x32\x10.service.GrpcKey2\xda\x02\n\x07Service\x12<\n\x07RunGame\x12\x17.service.RunGameRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x45\n\x08Spectate\x12\x18.service.SpectateRequest\x1a\x1b.service.SpectatorGameState\"\x00\x30\x01\x12G\n\nInitialize\x12\x1a.service.InitializeRequest\x1a\x1b.service.InitializeResponse\"\x00\x12H\n\x0bParticipate\x12\x1b.service.ParticipateRequest\x1a\x18.service.PlayerGameState\"\x00\x30\x01\x12\x37\n\x05Input\x12\x14.service.PlayerInput\x1a\x16.google.protobuf.Empty\"\x00\x42\x1e\n\x06protocB\x0cServiceProtoP\x00\xa2\x02\x03HLWb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'service_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006protocB\014ServiceProtoP\000\242\002\003HLW'
  _RUNGAMEREQUEST._serialized_start=82
  _RUNGAMEREQUEST._serialized_end=197
  _SPECTATEREQUEST._serialized_start=199
  _SPECTATEREQUEST._serialized_end=216
  _SPECTATORGAMESTATE._serialized_start=219
  _SPECTATORGAMESTATE._serialized_end=460
  _INITIALIZEREQUEST._serialized_start=462
  _INITIALIZEREQUEST._serialized_end=543
  _INITIALIZERESPONSE._serialized_start=545
  _INITIALIZERESPONSE._serialized_end=586
  _PARTICIPATEREQUEST._serialized_start=588
  _PARTICIPATEREQUEST._serialized_end=629
  _PLAYERGAMESTATE._serialized_start=632
  _PLAYERGAMESTATE._serialized_end=986
  _PLAYERINPUT._serialized_start=988
  _PLAYERINPUT._serialized_end=1059
  _SERVICE._serialized_start=1062
  _SERVICE._serialized_end=1408
# @@protoc_insertion_point(module_scope)



from .attack_data import AttackData
from .audio_data import AudioData
from .character_data import CharacterData
from .command_center import CommandCenter
from .fft_data import FftData
from .frame_data import FrameData
from .game_data import GameData
from .hit_area import HitArea
from .key import Key
from .round_result import RoundResult
from .screen_data import ScreenData



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: enum.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nenum.proto\x12\x07service*R\n\x08GrpcFlag\x12\t\n\x05\x45MPTY\x10\x00\x12\x0e\n\nINITIALIZE\x10\x01\x12\x0e\n\nPROCESSING\x10\x02\x12\r\n\tROUND_END\x10\x03\x12\x0c\n\x08GAME_END\x10\x04*M\n\tGrpcState\x12\x0f\n\x0bSTAND_STATE\x10\x00\x12\x10\n\x0c\x43ROUCH_STATE\x10\x01\x12\r\n\tAIR_STATE\x10\x02\x12\x0e\n\nDOWN_STATE\x10\x03*\xd9\x06\n\nGrpcAction\x12\x0b\n\x07NEUTRAL\x10\x00\x12\t\n\x05STAND\x10\x01\x12\x10\n\x0c\x46ORWARD_WALK\x10\x02\x12\x08\n\x04\x44\x41SH\x10\x03\x12\r\n\tBACK_STEP\x10\x04\x12\n\n\x06\x43ROUCH\x10\x05\x12\x08\n\x04JUMP\x10\x06\x12\x0c\n\x08\x46OR_JUMP\x10\x07\x12\r\n\tBACK_JUMP\x10\x08\x12\x07\n\x03\x41IR\x10\t\x12\x0f\n\x0bSTAND_GUARD\x10\n\x12\x10\n\x0c\x43ROUCH_GUARD\x10\x0b\x12\r\n\tAIR_GUARD\x10\x0c\x12\x15\n\x11STAND_GUARD_RECOV\x10\r\x12\x16\n\x12\x43ROUCH_GUARD_RECOV\x10\x0e\x12\x13\n\x0f\x41IR_GUARD_RECOV\x10\x0f\x12\x0f\n\x0bSTAND_RECOV\x10\x10\x12\x10\n\x0c\x43ROUCH_RECOV\x10\x11\x12\r\n\tAIR_RECOV\x10\x12\x12\x0f\n\x0b\x43HANGE_DOWN\x10\x13\x12\x08\n\x04\x44OWN\x10\x14\x12\x08\n\x04RISE\x10\x15\x12\x0b\n\x07LANDING\x10\x16\x12\x0b\n\x07THROW_A\x10\x17\x12\x0b\n\x07THROW_B\x10\x18\x12\r\n\tTHROW_HIT\x10\x19\x12\x10\n\x0cTHROW_SUFFER\x10\x1a\x12\x0b\n\x07STAND_A\x10\x1b\x12\x0b\n\x07STAND_B\x10\x1c\x12\x0c\n\x08\x43ROUCH_A\x10\x1d\x12\x0c\n\x08\x43ROUCH_B\x10\x1e\x12\t\n\x05\x41IR_A\x10\x1f\x12\t\n\x05\x41IR_B\x10 \x12\n\n\x06\x41IR_DA\x10!\x12\n\n\x06\x41IR_DB\x10\"\x12\x0c\n\x08STAND_FA\x10#\x12\x0c\n\x08STAND_FB\x10$\x12\r\n\tCROUCH_FA\x10%\x12\r\n\tCROUCH_FB\x10&\x12\n\n\x06\x41IR_FA\x10\'\x12\n\n\x06\x41IR_FB\x10(\x12\n\n\x06\x41IR_UA\x10)\x12\n\n\x06\x41IR_UB\x10*\x12\x11\n\rSTAND_D_DF_FA\x10+\x12\x11\n\rSTAND_D_DF_FB\x10,\x12\x11\n\rSTAND_F_D_DFA\x10-\x12\x11\n\rSTAND_F_D_DFB\x10.\x12\x11\n\rSTAND_D_DB_BA\x10/\x12\x11\n\rSTAND_D_DB_BB\x10\x30\x12\x0f\n\x0b\x41IR_D_DF_FA\x10\x31\x12\x0f\n\x0b\x41IR_D_DF_FB\x10\x32\x12\x0f\n\x0b\x41IR_F_D_DFA\x10\x33\x12\x0f\n\x0b\x41IR_F_D_DFB\x10\x34\x12\x0f\n\x0b\x41IR_D_DB_BA\x10\x35\x12\x0f\n\x0b\x41IR_D_DB_BB\x10\x36\x12\x11\n\rSTAND_D_DF_FC\x10\x37\x42\x1b\n\x06protocB\tEnumProtoP\x00\xa2\x02\x03HLWb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'enum_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\006protocB\tEnumProtoP\000\242\002\003HLW'
  _GRPCFLAG._serialized_start=23
  _GRPCFLAG._serialized_end=105
  _GRPCSTATE._serialized_start=107
  _GRPCSTATE._serialized_end=184
  _GRPCACTION._serialized_start=187
  _GRPCACTION._serialized_end=1044
# @@protoc_insertion_point(module_scope)



class Key:
    def __init__(self, key=None):
        if key:
            self.A: bool = key.A
            self.B: bool = key.B
            self.C: bool = key.C
            self.U: bool = key.U
            self.R: bool = key.R
            self.D: bool = key.D
            self.L: bool = key.L
        else:
            self.empty()
    
    def empty(self):
        self.A = False
        self.B = False
        self.C = False
        self.U = False
        self.R = False
        self.D = False
        self.L = False



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="pyftg",
    version="1.0.2",
    author="Staciiaz",
    description="An interface for implementing python AI in DareFightingICE",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Staciiaz/pyftg",
    python_requires=">=3.8",
    install_requires=["grpcio"],
    packages=find_packages(where='src'),
    package_dir={"": "src"},
    include_package_data=True,
)



from .struct import AudioData, ScreenData, FrameData, GameData, RoundResult, Key

class AIInterface:
    def name(self) -> str:
        pass

    def is_blind(self) -> bool:
        pass

    def initialize(self, game_data: GameData, player_number: bool):
        pass

    def get_information(self, frame_data: FrameData, is_control: bool, non_delay_frame_data: FrameData):
        pass

    def get_screen_data(self, screen_data: ScreenData):
        pass

    def get_audio_data(self, audio_data: AudioData):
        pass

    def processing(self):
        pass

    def input(self) -> Key:
        pass

    def round_end(self, round_result: RoundResult):
        pass

    def game_end(self):
        pass



from .key import Key
from .frame_data import FrameData

class CommandCenter:
    def __init__(self):
        self.skill_key: 'list[Key]' = []
        self.frame_data = FrameData()
        self.player_number = True

    def command_call(self, str):
        if not self.skill_key:
            self.action_to_command(str)

    def action_to_command(self, str: str):
        if str == "FORWARD_WALK":
            self.create_keys("6")
        elif str == "DASH":
            self.create_keys("6 5 6")
        elif str == "BACK_STEP":
            self.create_keys("4 5 4")
        elif str == "CROUCH":
            self.create_keys("2")
        elif str == "JUMP":
            self.create_keys("8")
        elif str == "FOR_JUMP":
            self.create_keys("9")
        elif str == "BACK_JUMP":
            self.create_keys("7")
        elif str == "STAND_GUARD":
            self.create_keys("4")
        elif str == "CROUCH_GUARD":
            self.create_keys("1")
        elif str == "AIR_GUARD":
            self.create_keys("7")
        elif str == "THROW_A":
            self.create_keys("4 _ A")
        elif str == "THROW_B":
            self.create_keys("4 _ B")
        elif str == "STAND_A":
            self.create_keys("A")
        elif str == "STAND_B":
            self.create_keys("B")
        elif str == "CROUCH_A":
            self.create_keys("2 _ A")
        elif str == "CROUCH_B":
            self.create_keys("2 _ B")
        elif str == "AIR_A":
            self.create_keys("A")
        elif str == "AIR_B":
            self.create_keys("B")
        elif str == "AIR_DA":
            self.create_keys("2 _ A")
        elif str == "AIR_DB":
            self.create_keys("2 _ B")
        elif str == "STAND_FA":
            self.create_keys("6 _ A")
        elif str == "STAND_FB":
            self.create_keys("6 _ B")
        elif str == "CROUCH_FA":
            self.create_keys("3 _ A")
        elif str == "CROUCH_FB":
            self.create_keys("3 _ B")
        elif str == "AIR_FA":
            self.create_keys("9 _ A")
        elif str == "AIR_FB":
            self.create_keys("9 _ B")
        elif str == "AIR_UA":
            self.create_keys("8 _ A")
        elif str == "AIR_UB":
            self.create_keys("8 _ B")
        elif str == "STAND_D_DF_FA":
            self.create_keys("2 3 6 _ A")
        elif str == "STAND_D_DF_FB":
            self.create_keys("2 3 6 _ B")
        elif str == "STAND_F_D_DFA":
            self.create_keys("6 2 3 _ A")
        elif str == "STAND_F_D_DFB":
            self.create_keys("6 2 3 _ B")
        elif str == "STAND_D_DB_BA":
            self.create_keys("2 1 4 _ A")
        elif str == "STAND_D_DB_BB":
            self.create_keys("2 1 4 _ B")
        elif str == "AIR_D_DF_FA":
            self.create_keys("2 3 6 _ A")
        elif str == "AIR_D_DF_FB":
            self.create_keys("2 3 6 _ B")
        elif str == "AIR_F_D_DFA":
            self.create_keys("6 2 3 _ A")
        elif str == "AIR_F_D_DFB":
            self.create_keys("6 2 3 _ B")
        elif str == "AIR_D_DB_BA":
            self.create_keys("2 1 4 _ A")
        elif str == "AIR_D_DB_BB":
            self.create_keys("2 1 4 _ B")
        elif str == "STAND_D_DF_FC":
            self.create_keys("2 3 6 _ C")
        else:
            self.create_keys(str)

    def create_keys(self, str: str):
        buf = None
        commands = str.split(" ")
        if not self.frame_data.is_front(self.player_number):
            commands = self.reverse_key(commands)

        index = 0
        while index < len(commands):
            buf = Key()
            if commands[index] == "L" or commands[index] == "4":
                buf.L = True
            elif commands[index] == "R" or commands[index] == "6":
                buf.R = True
            elif commands[index] == "D" or commands[index] == "2":
                buf.D = True
            elif commands[index] == "U" or commands[index] == "8":
                buf.U = True
            elif commands[index] == "LD" or commands[index] == "1":
                buf.L = True
                buf.D = True
            elif commands[index] == "LU" or commands[index] == "7":
                buf.L = True
                buf.U = True
            elif commands[index] == "RD" or commands[index] == "3":
                buf.R = True
                buf.D = True
            elif commands[index] == "RU" or commands[index] == "9":
                buf.R = True
                buf.U = True

            if index + 2 < len(commands) and commands[index + 1] == "_":
                index += 2
            if commands[index] == "A":
                buf.A = True
            elif commands[index] == "B":
                buf.B = True
            elif commands[index] == "C":
                buf.C = True
            self.skill_key.append(buf)
            index += 1

    def set_frame_data(self, frame_data: FrameData, player_number: bool):
        self.frame_data = frame_data
        self.player_number = player_number

    def get_skill_flag(self) -> bool:
        return len(self.skill_key) > 0

    def get_skill_key(self) -> Key:
        if self.get_skill_flag():
            return self.skill_key.pop(0)
        else:
            return Key()

    def get_skill_keys(self) -> 'list[Key]':
        return self.skill_key

    def skill_cancel(self):
        self.skill_key.clear()

    def is_player_number(self) -> bool:
        return self.player_number

    def reverse_key(self, commands: 'list[str]'):
        buffer = [0]*len(commands)
        for i in range(len(commands)):
            if commands[i] == "L" or commands[i] == "4":
                buffer[i] = "6"
            elif commands[i] == "R" or commands[i] == "6":
                buffer[i] = "4"
            elif commands[i] == "LD" or commands[i] == "1":
                buffer[i] = "3"
            elif commands[i] == "LU" or commands[i] == "7":
                buffer[i] = "9"
            elif commands[i] == "RD" or commands[i] == "3":
                buffer[i] = "1"
            elif commands[i] == "RU" or commands[i] == "9":
                buffer[i] = "7"
            else:
                buffer[i] = commands[i]
        return buffer
